import Tokenizer from './infra/tokenizer.js';
import TunaErrorHandler from "./infra/tunaErrorHandler.js";
import Configs from "./config.js";
import antifraudScripts from "./antifraudScripts.js";

export class Essentials {

  constructor(sessionID, env, antifraudConfig) {
    if (!sessionID)
      throw new TunaErrorHandler("ERR:34");
    this.env = env;
    this.sessionID = sessionID;
    this.antifraudConfig = antifraudConfig;
    initAntiFrauds(this, antifraudConfig);
  }

  getSessionID = _ => this.sessionID;

  tokenizator = _ => {
    if (!this.tokenizer)
      this.tokenizer = new Tokenizer(this.sessionID, this.env);
    return this.tokenizer;
  }

  async getCheckoutData(creditCardData, resultCallback) {
    const isResponseByCallback = resultCallback && typeof resultCallback === "function";

    //Create a safe copy of the credit card data object
    let safeCreditCardData = { ...creditCardData };
    safeCreditCardData.maskedNumber = maskCreditCard(safeCreditCardData.cardNumber);
    delete safeCreditCardData["cvv"];
    delete safeCreditCardData["cardNumber"];

    const tokenResponse = await this.tokenizator().generate(creditCardData);
    const result = { success: tokenResponse?.code === 1, tokenData: tokenResponse, cardData: safeCreditCardData };
    if (isResponseByCallback) {
      resultCallback(result);
      return;
    }
    return result;
  }

  async pay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, checkoutData, customer) {

    let paymentMethod = {
      Amount: paymentMethodAmount
    };

    switch (paymentMethodType) {
      case "creditCard":

        if (!checkoutData)
          throw new TunaErrorHandler("ERR:37");

        if (!customer)
          customer = checkoutData?.cardData?.customer;

        const { cardData, tokenData } = checkoutData;

        try {
          paymentMethod.PaymentMethodType = '1';
          paymentMethod.Installments = cardData.installment || 1;
          paymentMethod.CardInfo = {
            TokenProvider: "Tuna",
            Token: tokenData.token,
            BrandName: tokenData.brand,
            SaveCard: !cardData.singleUse,
            ExpirationMonth: cardData.expirationMonth,
            ExpirationYear: cardData.expirationYear,
            CardHolderName: cardData.cardHolderName,
            CardNumber: cardData.maskedNumber.trim()
          };
          if (tokenData.authenticationInformation) {
            paymentMethod.AuthenticationInformation = {
              Code: this.sessionID,
              ReferenceId: tokenData.authenticationInformation.referenceId,
              TransactionId: tokenData.authenticationInformation.transactionId
            }
          }
        } catch {
          throw new TunaErrorHandler("ERR:42");
        }
        break;
      case "pix":
        paymentMethod.PaymentMethodType = 'D';
        break;
      case "crypto":
        paymentMethod.PaymentMethodType = 'E';
        break;
      case "bankInvoice":
        paymentMethod.PaymentMethodType = '3';
        break;
      default:
        throw new TunaErrorHandler("ERR:36");
    }

    let initRequest = {
      TokenSession: this.getSessionID(),
      customer,
      PaymentData: {
        Amount: totalPaymentAmount,
        CountryCode: "BR",
        PaymentMethods: [paymentMethod]
      },
    };
    return await callInit(this, initRequest);
  }

  async doStatusLongPolling(callbackFunction, methodID, paymentKey) {
    try {
      const successStatuses = ['2', '8'];
      const failStatuses = ['4', '5', 'A', 'N'];

      const options = {
        method: 'post',
        headers: {
          'Content-type': 'application/json; charset=UTF-8',
          'x-tuna-token-session': this.getSessionID()
        },
        body: JSON.stringify({ MethodID: methodID, PaymentStatusList: [...successStatuses, ...failStatuses], PaymentKey: paymentKey })
      }

      let response = await (await fetch(`${Configs[this.env].INTEGRATIONS_API_URL}/StatusPoll`, options)).json();

      if (response.paymentMethodConfimed) {

        response.paymentApproved = false;

        if (successStatuses.includes(response.paymentStatusFound))
          response.paymentApproved = true;

        callbackFunction(response);
      }
      else if (response.allowRetry)
        await this.doStatusLongPolling(callbackFunction, methodID, paymentKey);
      else {
        response.paymentApproved = false;
        callbackFunction(response);
      }

    } catch (error) {
      callbackFunction({ code: -1, paymentMethodConfimed: false, allowRetry: false }, error);
    }
  }
}

function maskCreditCard(creditCardNumber) {
  if (!creditCardNumber || (creditCardNumber && typeof creditCardNumber !== "string"))
    return null;

  // clear formatting mask
  creditCardNumber = creditCardNumber.replace(/[^\da-zA-Z]/g, '');

  let maskedCreditCard = creditCardNumber.substring(0, 6) + "xxxxxx" + creditCardNumber.slice(-4);
  return maskedCreditCard;
}

async function callInit(tunaObject, initRequest) {
  try {
    const options = {
      method: 'post',
      headers: {
        'Content-type': 'application/json; charset=UTF-8',
        'x-tuna-token-session': tunaObject.sessionID,
      },
      body: JSON.stringify(initRequest)
    }
    return await (await fetch(`${Configs[tunaObject.env].INTEGRATIONS_API_URL}/Init`, options)).json();

  } catch (error) {
    throw new TunaErrorHandler(error);
  }
}

function avaliableAntifrauds() {
  return ["clearsale", "siftscience", "konduto", "cybersource"];
}

async function initAntiFrauds(tunaObject, antifraudParameters) {
  if (antifraudParameters) {
    try {

      var antifraudsEnabled = antifraudParameters.filter(config => avaliableAntifrauds().includes(config.key.toLowerCase()));

      antifraudsEnabled.forEach(antifraudConfig => {

        antifraudScripts[antifraudConfig.key.toLowerCase()].forEach(script => {

          let tag = script.tag.replace("#key#", antifraudConfig.value);

          if (antifraudConfig.key.toLowerCase() === "cybersource") {
            tag = tag.replace("#orgid#", Configs[tunaObject.env].CYBERSOURCE_ORG_ID);
          }

          antifraudParameters.forEach(config => {
            tag = tag.replaceAll(`#${config.key}#`, config.value);
          });

          const parser = Range.prototype.createContextualFragment.bind(document.createRange());
          document.getElementsByTagName(script.location)[0].appendChild(parser(tag));
        });

      });

    } catch (error) {
      throw new TunaErrorHandler("ERR:40");
    }
  }
}

export default function factory(sessionID, env = "production", antifraudConfig = null) {
  if (sessionID)
    return new Essentials(sessionID, env, antifraudConfig);
  else
    throw new TunaErrorHandler("ERR:34");
}
