import TunaErrorHandler from "./infra/tunaErrorHandler.js";
import { Essentials } from "./essentials.js";
import PieceManager from "./dom/pieceManager.js";
import { BuyerDocumentFormatter, BrazilianDocumentFormatter } from "./infra/buyerDocumentFormatter.js";
import GooglePayResolver from "./infra/googlePayResolver.js"

class Tuna extends Essentials {

  constructor(sessionID, env, antifraudConfig) {
    super(sessionID, env, antifraudConfig);
    this.isCheckoutRunning = false;
    this.pieceManagerObject = new PieceManager();
  }

  getBuyerDocumentFormatter = locale => locale === "pt-BR" ? new BrazilianDocumentFormatter() : new BuyerDocumentFormatter();
  close3dsModal = _ => this.pieceManager().close3dsModal();
  pieceManager = _ => this.pieceManagerObject;
  clearSavedCardSelector = _ => this.pieceManager().executeOnPieces(piece => piece.getType() === this.pieceManager().SAVED_CARD_LIST ? piece.clear() : '');

  useGooglePay(googlePayBtnContainerSelector, googlePaySettings, checkoutCallbackFunction, validationGroup) {
    if (!("cardAuthMethods" in googlePaySettings)
      || !("cardNetworks" in googlePaySettings)
      || !("gatewayParameters" in googlePaySettings))
      throw new TunaErrorHandler("ERR:19");

    if (!checkoutCallbackFunction || typeof checkoutCallbackFunction !== "function")
      throw new TunaErrorHandler("ERR:09");

    if (document.querySelector(googlePayBtnContainerSelector)) {

      const onGooglePayButtonClicked = validationGroup => {
        let validationGroupArray = [];
        if (validationGroup) {
          if (typeof validationGroup === "string")
            validationGroupArray = [validationGroup];
          else if (Array.isArray(validationGroup))
            validationGroupArray = validationGroup
        }

        this.isCheckoutRunning = true;
        try {
          let screenFields;
          if (validationGroupArray.length > 0) {
            screenFields = getValuesFromPieces(this, validationGroupArray);
          }
          setTimeout(_ => this.isCheckoutRunning = false, 500);
          return screenFields;
        } catch (e) {
          this.isCheckoutRunning = false;
          throw e;
        }
      }

      this.googlePayResolver = new GooglePayResolver(
        googlePaySettings,
        _ => onGooglePayButtonClicked(validationGroup),
        checkoutCallbackFunction);

      this.googlePayResolver.init(googlePayBtnContainerSelector);
    }
    else
      throw new TunaErrorHandler("ERR:18", { selector: googlePayBtnContainerSelector });
  }

  forgeDefaultForm(place, options) {

    if (!options.checkoutCallback && !options.checkoutAndPayConfig)
      throw new TunaErrorHandler("ERR:25");

    if (options.checkoutAndPayConfig && typeof options.checkoutAndPayConfig !== "object")
      throw new TunaErrorHandler("ERR:26");

    if (options.checkoutAndPayConfig) {
      try {
        if (typeof options.checkoutAndPayConfig.totalPaymentAmount !== "number" || options.checkoutAndPayConfig.totalPaymentAmount <= 0)
          throw new TunaErrorHandler("ERR:27");
        if (typeof options.checkoutAndPayConfig.paymentMethodAmount !== "number" || options.checkoutAndPayConfig.paymentMethodAmount <= 0)
          throw new TunaErrorHandler("ERR:28");
        if (typeof options.checkoutAndPayConfig.callbackFunction !== "function")
          throw new TunaErrorHandler("ERR:29");
      } catch {
        throw new TunaErrorHandler("ERR:30");
      }

      if (options.customAreas) {
        if (!Array.isArray(options.customAreas))
          options.customAreas = [options.customAreas];

        options.customAreas.forEach(customArea => {
          if (!customArea.title)
            throw new TunaErrorHandler("ERR:38");
          if (!customArea.fields)
            throw new TunaErrorHandler("ERR:39");
        });
      }

    }

    this.pieceManager().forge(place, "defaultForm",
      {
        interfaceManager: this,
        pieceManager: this.pieceManager(),
        options: options || {}
      });
  }

  useQrCodePayment(place, options, paymentCallback, group = "default") {

    if (!paymentCallback || typeof paymentCallback !== "function")
      throw new TunaErrorHandler("ERR:24");

    if (!options || options.methodID === "undefined" || typeof options.methodID !== "number")
      throw new TunaErrorHandler("ERR:32", { propName: "methodID" });

    if (!options.paymentKey || typeof options.paymentKey !== "string")
      throw new TunaErrorHandler("ERR:32", { propName: "paymentKey" });

    this.pieceManager().forge(place, "qrCodePaymentBox", options, group);

    this.doStatusLongPolling(paymentCallback, options.methodID, options.paymentKey);
  }

  async useSavedCardSelector(place, options, group = "default") {
    let skeletonID;
    skeletonID = this.pieceManager().forge(place, "savedCardSkeleton");

    let response = await this.tokenizator().list();
    if (response && response.code === 1 && response.tokens && response.tokens.length > 0) {
      if (skeletonID)
        this.pieceManager().destroy(skeletonID);

      this.pieceManager().forge(place, "savedCardSelector", {
        tokens: response.tokens,
        pieceManager: this.pieceManager(),
        ...options
      }, group);

      return response.tokens.length;
    } else {
      if (skeletonID)
        this.pieceManager().destroy(skeletonID);
      return 0;
    }
  }

  async checkoutAndPay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, callbackFunction, customer) {

    if (this.isCheckoutRunning)
      throw new TunaErrorHandler("ERR:14");

    if (!callbackFunction || (callbackFunction && typeof callbackFunction !== "function"))
      throw new TunaErrorHandler("ERR:09");

    try {
      this.isCheckoutRunning = true;
      let checkoutData = null;

      if (paymentMethodType === "creditCard") {

        checkoutData = await prepareCheckout(this);
        
        if (!checkoutData.success) {
          this.isCheckoutRunning = false;
          callbackFunction({ success: false, message: checkoutData.message });
          return;
        }
        const authenticationInformation = checkoutData?.tokenData?.authenticationInformation;

        if (authenticationInformation && authenticationInformation.accessToken && authenticationInformation.deviceDataCollectionUrl) {
          this.create3DSDataCollectionFrame(authenticationInformation, async status => {
            if (status === true) {
              try {
                const paymentResponse = await this.pay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, checkoutData, customer);
                callbackFunction({ success: true, data: paymentResponse });
              } catch (e) {
                callbackFunction({ success: false, message: e.message });
              }
            }
            else
              callbackFunction({ success: false, message: new TunaErrorHandler("ERR:41").message });
            this.isCheckoutRunning = false;
          });
          return;
        }
      }

      const paymentResponse = await this.pay(paymentMethodType, totalPaymentAmount, paymentMethodAmount, checkoutData, customer);
      callbackFunction({ success: true, data: paymentResponse });
      this.isCheckoutRunning = false;
    } catch (e) {
      this.isCheckoutRunning = false;
      callbackFunction({ success: false, message: e.message });
    }
  }

  async checkout(callbackFunction) {
    if (this.isCheckoutRunning)
      throw new TunaErrorHandler("ERR:14");

    this.isCheckoutRunning = true;

    let prepearedCheckout;
    try {
      prepearedCheckout = await prepareCheckout(this);
    } finally {
      this.isCheckoutRunning = false;
    }

    if (callbackFunction && typeof callbackFunction === "function") {
      callbackFunction(prepearedCheckout);
      return;
    }
    return prepearedCheckout;
  }

  open3dsModal(threeDSInfo, methodID, paymentKey, responseCallback) {
    const elemDiv = document.createElement('div');
    const divID = `tds_${Math.random().toString(18).substring(2)}`;
    elemDiv.id = divID;
    document.body.appendChild(elemDiv);
    this.pieceManager().forge(`#${divID}`, "threeDSModal", { threeDSInfo });
    this.doStatusLongPolling(responseCallback, methodID, paymentKey);
  }

  create3DSDataCollectionFrame({ accessToken, deviceDataCollectionUrl }, onDataCollectionScriptReady) {
    const elemDiv = document.createElement('div');
    const divID = `tds_${Math.random().toString(18).substring(2)}`;
    elemDiv.id = divID;
    document.body.appendChild(elemDiv);
    this.pieceManager().forge(`#${divID}`, "threeDSDataCollectionFrame", { accessToken, deviceDataCollectionUrl });

    window.addEventListener("message", async event => {
      if (event.origin === "https://centinelapistag.cardinalcommerce.com")
        onDataCollectionScriptReady(JSON.parse(event.data).Status);
    }, false);
  }

  //DEPRECATED
  directCheckout = async _ => {
    console.warn("The directCheckout function is deprecated. Use checkout() function instead");
    return await this.checkout();
  };
}

async function prepareCheckout(tunaObject) {

  let resultingPiecesValues = getValuesFromPieces(tunaObject);

  if (!resultingPiecesValues)
    throw new TunaErrorHandler("ERR:16");

  const getCheckoutDataFromPiecesValues = async resultingPiecesValues => {
    if (resultingPiecesValues.savedCard != null) {
      let savedCard = resultingPiecesValues.savedCard;

      let bindedToken = await tunaObject.tokenizator().bind(savedCard.token, savedCard.cvv);

      if (!bindedToken || bindedToken.code !== 1)
        throw new TunaErrorHandler("ERR:17");

      let tokenData = { brand: savedCard.brand, token: savedCard.token, code: 1 };

      delete savedCard.brand;
      delete savedCard.token;
      delete savedCard.cvv;

      delete resultingPiecesValues.savedCard
      delete resultingPiecesValues.cvv;

      return {
        success: true,
        tokenData: {
          ...tokenData,
          authenticationInformation: bindedToken.authenticationInformation
        },
        cardData: {
          ...resultingPiecesValues,
          ...savedCard
        }
      };
    }
    return null;
  };

  return await getCheckoutDataFromPiecesValues(resultingPiecesValues) || await tunaObject.getCheckoutData(resultingPiecesValues);
}

function getValuesFromPieces(tunaObject, validationGroupArray = null) {
  let response = {};
  let invalidFieldPropertyNames = [];

  tunaObject.pieceManager().executeOnPieces(piece => {

    if (validationGroupArray === null || (Array.isArray(validationGroupArray) && validationGroupArray.includes(piece.getGroup()))) {
      let pieceType = piece.getType();
      if (pieceType.propertyName && piece.isEnabled) {
        try {
          let value = piece.getValue();

          if (pieceType.propertyName === "expiration" && value) {
            response = {
              ...response,
              "expirationYear": value.expirationYear,
              "expirationMonth": value.expirationMonth
            };
          } else if (piece.opts.resultingObject) {
            response = { ...response, [piece.opts.resultingObject]: { ...response[piece.opts.resultingObject], [pieceType.propertyName]: value } };
          } else
            response = { ...response, [pieceType.propertyName]: value };

        } catch {
          invalidFieldPropertyNames.push(pieceType.propertyName);
        }
      } else if (!piece.isEnabled) {
        piece.resetInvalidMarkup();
      }
    }
  });

  if (invalidFieldPropertyNames.length > 0)
    throw new TunaErrorHandler("ERR:07", { fieldNames: invalidFieldPropertyNames.join(", ") });

  if (response.singleUse !== undefined) {
    // At screen, the common is use "save card" checkbox. Once the tokenization expects a "single use" flag, the value need to be inverted
    response.singleUse = !response.singleUse;
  }

  return response;
}

export default function factory(sessionID, env = "production", antifraudConfig = null) {
  if (sessionID)
    return new Tuna(sessionID, env, antifraudConfig);
  else
    throw new TunaErrorHandler("ERR:34");
}